@@ -103,7 +103,8 @@ static void gst_ffmpegmux_init (GstFFMpegMux * ffmpegmux,
     GstFFMpegMuxClass * g_class);
 static void gst_ffmpegmux_finalize (GObject * object);
 
-static gboolean gst_ffmpegmux_setcaps (GstPad * pad, GstCaps * caps);
+static gboolean gst_ffmpegmux_setcaps (GstPad * pad, GstObject * parent,
+    GstCaps * caps);
 static GstPad *gst_ffmpegmux_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * name, const GstCaps * caps);
 static GstFlowReturn gst_ffmpegmux_collected (GstCollectPads * pads,
@@ -160,7 +161,7 @@ gst_ffmpegmux_get_replacement (const char *name)
     {"mp3", "id3v2mux"},
     {"mp2", "id3v2mux"}
   };
-  int i;
+  guint i;
 
   for (i = 0; i < sizeof (blacklist) / sizeof (blacklist[0]); i++) {
     if (strcmp (blacklist[i].name, name) == 0) {
@@ -259,11 +260,6 @@ gst_ffmpegmux_base_init (gpointer g_class)
     gst_caps_set_simple (audiosinkcaps,
         "rate", G_TYPE_INT, 48000, "channels", G_TYPE_INT, 2, NULL);
 
-  } else if (strcmp (in_plugin->name, "gif") == 0) {
-    if (videosinkcaps)
-      gst_caps_unref (videosinkcaps);
-
-    videosinkcaps = gst_caps_from_string ("video/x-raw, format=(string)RGB");
   }
 
   /* pad templates */
@@ -451,10 +447,10 @@ gst_ffmpegmux_request_new_pad (GstElement * element,
   /* AVStream needs to be created */
   st = avformat_new_stream (ffmpegmux->context, NULL);
   st->id = collect_pad->padnum;
-  st->codec->codec_type = type;
-  st->codec->codec_id = AV_CODEC_ID_NONE;       /* this is a check afterwards */
-  st->codec->bit_rate = bitrate;
-  st->codec->frame_size = framesize;
+  st->codecpar->codec_type = type;
+  st->codecpar->codec_id = AV_CODEC_ID_NONE;    /* this is a check afterwards */
+  st->codecpar->bit_rate = bitrate;
+  st->codecpar->frame_size = framesize;
   /* we fill in codec during capsnego */
 
   /* we love debug output (c) (tm) (r) */
@@ -465,36 +461,31 @@ gst_ffmpegmux_request_new_pad (GstElement * element,
   return pad;
 }
 
-/**
- * gst_ffmpegmux_setcaps
- * @pad: #GstPad
- * @caps: New caps.
- *
- * Set caps to pad.
- *
- * Returns: #TRUE on success.
- */
 static gboolean
-gst_ffmpegmux_setcaps (GstPad * pad, GstCaps * caps)
+gst_ffmpegmux_setcaps (GstPad * pad, GstObject * parent, GstCaps * caps)
 {
-  GstFFMpegMux *ffmpegmux = (GstFFMpegMux *) (gst_pad_get_parent (pad));
+  GstFFMpegMux *ffmpegmux = (GstFFMpegMux *) parent;
   GstFFMpegMuxPad *collect_pad;
   AVStream *st;
+  AVCodecContext tmp;
 
   collect_pad = (GstFFMpegMuxPad *) gst_pad_get_element_private (pad);
 
   st = ffmpegmux->context->streams[collect_pad->padnum];
   av_opt_set_int (ffmpegmux->context, "preload", ffmpegmux->preload, 0);
   ffmpegmux->context->max_delay = ffmpegmux->max_delay;
+  memset (&tmp, 0, sizeof (tmp));
 
   /* for the format-specific guesses, we'll go to
    * our famous codec mapper */
-  if (gst_ffmpeg_caps_to_codecid (caps, st->codec) == AV_CODEC_ID_NONE)
+  if (gst_ffmpeg_caps_to_codecid (caps, &tmp) == AV_CODEC_ID_NONE)
     goto not_accepted;
 
+  avcodec_parameters_from_context (st->codecpar, &tmp);
+
   /* copy over the aspect ratios, ffmpeg expects the stream aspect to match the
    * codec aspect. */
-  st->sample_aspect_ratio = st->codec->sample_aspect_ratio;
+  st->sample_aspect_ratio = st->codecpar->sample_aspect_ratio;
 
   GST_LOG_OBJECT (pad, "accepted caps %" GST_PTR_FORMAT, caps);
   return TRUE;
@@ -527,7 +518,7 @@ gst_ffmpegmux_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
     case GST_EVENT_CAPS:{
       GstCaps *caps;
       gst_event_parse_caps (event, &caps);
-      if (!(res = gst_ffmpegmux_setcaps (pad, caps)))
+      if (!(res = gst_ffmpegmux_setcaps (pad, parent, caps)))
         goto beach;
       break;
     }
@@ -568,23 +559,23 @@ gst_ffmpegmux_collected (GstCollectPads * pads, gpointer user_data)
       AVStream *st = ffmpegmux->context->streams[collect_pad->padnum];
 
       /* check whether the pad has successfully completed capsnego */
-      if (st->codec->codec_id == AV_CODEC_ID_NONE) {
+      if (st->codecpar->codec_id == AV_CODEC_ID_NONE) {
         GST_ELEMENT_ERROR (ffmpegmux, CORE, NEGOTIATION, (NULL),
             ("no caps set on stream %d (%s)", collect_pad->padnum,
-                (st->codec->codec_type == AVMEDIA_TYPE_VIDEO) ?
+                (st->codecpar->codec_type == AVMEDIA_TYPE_VIDEO) ?
                 "video" : "audio"));
         return GST_FLOW_ERROR;
       }
       /* set framerate for audio */
-      if (st->codec->codec_type == AVMEDIA_TYPE_AUDIO) {
-        switch (st->codec->codec_id) {
+      if (st->codecpar->codec_type == AVMEDIA_TYPE_AUDIO) {
+        switch (st->codecpar->codec_id) {
           case AV_CODEC_ID_PCM_S16LE:
           case AV_CODEC_ID_PCM_S16BE:
           case AV_CODEC_ID_PCM_U16LE:
           case AV_CODEC_ID_PCM_U16BE:
           case AV_CODEC_ID_PCM_S8:
           case AV_CODEC_ID_PCM_U8:
-            st->codec->frame_size = 1;
+            st->codecpar->frame_size = 1;
             break;
           default:
           {
@@ -595,8 +586,8 @@ gst_ffmpegmux_collected (GstCollectPads * pads, gpointer user_data)
             buffer = gst_collect_pads_peek (ffmpegmux->collect,
                 (GstCollectData *) collect_pad);
             if (buffer) {
-              st->codec->frame_size =
-                  st->codec->sample_rate *
+              st->codecpar->frame_size =
+                  st->codecpar->sample_rate *
                   GST_BUFFER_DURATION (buffer) / GST_SECOND;
               gst_buffer_unref (buffer);
             }
@@ -730,49 +721,23 @@ gst_ffmpegmux_collected (GstCollectPads * pads, gpointer user_data)
    * no buffers left */
   if (best_pad != NULL) {
     GstBuffer *buf;
-    AVPacket pkt;
-    gboolean need_free = FALSE;
+    AVPacket pkt = { 0, };
     GstMapInfo map;
 
     /* push out current buffer */
     buf =
         gst_collect_pads_pop (ffmpegmux->collect, (GstCollectData *) best_pad);
 
-    ffmpegmux->context->streams[best_pad->padnum]->codec->frame_number++;
-
     /* set time */
     pkt.pts = gst_ffmpeg_time_gst_to_ff (GST_BUFFER_TIMESTAMP (buf),
         ffmpegmux->context->streams[best_pad->padnum]->time_base);
     pkt.dts = pkt.pts;
 
-    if (strcmp (ffmpegmux->context->oformat->name, "gif") == 0) {
-      AVStream *st = ffmpegmux->context->streams[best_pad->padnum];
-      AVPicture src, dst;
-
-      need_free = TRUE;
-      pkt.size = st->codec->width * st->codec->height * 3;
-      pkt.data = g_malloc (pkt.size);
-
-      dst.data[0] = pkt.data;
-      dst.data[1] = NULL;
-      dst.data[2] = NULL;
-      dst.linesize[0] = st->codec->width * 3;
-
-      gst_buffer_map (buf, &map, GST_MAP_READ);
-      gst_ffmpeg_avpicture_fill (&src, map.data,
-          AV_PIX_FMT_RGB24, st->codec->width, st->codec->height);
-
-      av_picture_copy (&dst, &src, AV_PIX_FMT_RGB24,
-          st->codec->width, st->codec->height);
-      gst_buffer_unmap (buf, &map);
-    } else {
-      gst_buffer_map (buf, &map, GST_MAP_READ);
-      pkt.data = map.data;
-      pkt.size = map.size;
-    }
+    gst_buffer_map (buf, &map, GST_MAP_READ);
+    pkt.data = map.data;
+    pkt.size = map.size;
 
     pkt.stream_index = best_pad->padnum;
-    pkt.flags = 0;
 
     if (!GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DELTA_UNIT))
       pkt.flags |= AV_PKT_FLAG_KEY;
@@ -781,14 +746,8 @@ gst_ffmpegmux_collected (GstCollectPads * pads, gpointer user_data)
       pkt.duration =
           gst_ffmpeg_time_gst_to_ff (GST_BUFFER_DURATION (buf),
           ffmpegmux->context->streams[best_pad->padnum]->time_base);
-    else
-      pkt.duration = 0;
     av_write_frame (ffmpegmux->context, &pkt);
-    if (need_free) {
-      g_free (pkt.data);
-    } else {
-      gst_buffer_unmap (buf, &map);
-    }
+    gst_buffer_unmap (buf, &map);
     gst_buffer_unref (buf);
   } else {
     /* close down */
@@ -871,7 +830,7 @@ gst_ffmpeg_mux_simple_caps_set_int_list (GstCaps * caps, const gchar * field,
 {
   GValue list = { 0, };
   GValue val = { 0, };
-  gint i;
+  guint i;
 
   g_return_if_fail (GST_CAPS_IS_SIMPLE (caps));
 
@@ -907,13 +866,12 @@ gst_ffmpegmux_register (GstPlugin * plugin)
     NULL, NULL, NULL
   };
   GType type;
-  AVOutputFormat *in_plugin;
-
-  in_plugin = av_oformat_next (NULL);
+  const AVOutputFormat *in_plugin;
+  void *i = 0;
 
   GST_LOG ("Registering muxers");
 
-  while (in_plugin) {
+  while ((in_plugin = av_muxer_iterate (&i))) {
     gchar *type_name;
     GstRank rank = GST_RANK_MARGINAL;
 
@@ -942,6 +900,7 @@ gst_ffmpegmux_register (GstPlugin * plugin)
         (!strncmp (in_plugin->name, "ffmetadata", 10)) ||
         (!strncmp (in_plugin->name, "srt", 3)) ||
         (!strncmp (in_plugin->name, "scc", 3)) ||
+        !strcmp (in_plugin->name, "ttml") ||
         !strcmp (in_plugin->name, "segment") ||
         !strcmp (in_plugin->name, "stream_segment,ssegment") ||
         !strcmp (in_plugin->name, "jacosub") ||
@@ -952,12 +911,14 @@ gst_ffmpegmux_register (GstPlugin * plugin)
         !strncmp (in_plugin->name, "webm", 4)
         ) {
       GST_LOG ("Ignoring muxer %s", in_plugin->name);
-      goto next;
+      continue;
     }
 
-    if ((!strncmp (in_plugin->long_name, "raw ", 4))) {
-      GST_LOG ("Ignoring raw muxer %s", in_plugin->name);
-      goto next;
+    if (in_plugin->long_name != NULL) {
+      if ((!strncmp (in_plugin->long_name, "raw ", 4))) {
+        GST_LOG ("Ignoring raw muxer %s", in_plugin->name);
+        continue;
+      }
     }
 
     if (gst_ffmpegmux_get_replacement (in_plugin->name))
@@ -985,9 +946,6 @@ gst_ffmpegmux_register (GstPlugin * plugin)
     }
 
     g_free (type_name);
-
-  next:
-    in_plugin = av_oformat_next (in_plugin);
   }
 
   GST_LOG ("Finished registering muxers");
